// DemoDoc.cpp : implementation of the CDemoDoc class
//

#include "stdafx.h"
#include "MultiRectTracker_Demo.h"

#include "DemoDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




// LoadBMPImage - Loads a BMP file and creates a bitmap GDI object
//                also creates logical palette for it.
// Returns      - TRUE for success
// sBMPFile     - Full path of the BMP file
// bitmap       - The bitmap object to initialize
// pPal         - Will hold the logical palette. Can be NULL
BOOL LoadBMPImage( LPCTSTR sBMPFile, CBitmap& bitmap, CPalette *pPal )
{
        CFile file;
        if( !file.Open( sBMPFile, CFile::modeRead) )
                return FALSE;

        BITMAPFILEHEADER bmfHeader;

        // Read file header
        if (file.Read((LPSTR)&bmfHeader, sizeof(bmfHeader)) != sizeof(bmfHeader))
                return FALSE;

        // File type should be 'BM'
        if (bmfHeader.bfType != ((WORD) ('M' << 8) | 'B'))
                return FALSE;

        // Get length of the remainder of the file and allocate memory
        DWORD nPackedDIBLen = file.GetLength() - sizeof(BITMAPFILEHEADER);
        HGLOBAL hDIB = ::GlobalAlloc(GMEM_FIXED, nPackedDIBLen);
        if (hDIB == 0)
                return FALSE;

        // Read the remainder of the bitmap file.
        if (file.ReadHuge((LPSTR)hDIB, nPackedDIBLen) != nPackedDIBLen )
        {
                ::GlobalFree(hDIB);
                return FALSE;
        }


        BITMAPINFOHEADER &bmiHeader = *(LPBITMAPINFOHEADER)hDIB ;
        BITMAPINFO &bmInfo = *(LPBITMAPINFO)hDIB ;

        // If bmiHeader.biClrUsed is zero we have to infer the number
        // of colors from the number of bits used to specify it.
        int nColors = bmiHeader.biClrUsed ? bmiHeader.biClrUsed : 
                                                1 << bmiHeader.biBitCount;

        LPVOID lpDIBBits;
        if( bmInfo.bmiHeader.biBitCount > 8 )
                lpDIBBits = (LPVOID)((LPDWORD)(bmInfo.bmiColors + bmInfo.bmiHeader.biClrUsed) + 
                        ((bmInfo.bmiHeader.biCompression == BI_BITFIELDS) ? 3 : 0));
        else
                lpDIBBits = (LPVOID)(bmInfo.bmiColors + nColors);

        // Create the logical palette
        if( pPal != NULL )
        {
                // Create the palette
                if( nColors <= 256 )
                {
                        UINT nSize = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * nColors);
                        LOGPALETTE *pLP = (LOGPALETTE *) new BYTE[nSize];

                        pLP->palVersion = 0x300;
                        pLP->palNumEntries = nColors;

                        for( int i=0; i < nColors; i++)
                        {
                                pLP->palPalEntry[i].peRed = bmInfo.bmiColors[i].rgbRed;
                                pLP->palPalEntry[i].peGreen = bmInfo.bmiColors[i].rgbGreen;
                                pLP->palPalEntry[i].peBlue = bmInfo.bmiColors[i].rgbBlue;
                                pLP->palPalEntry[i].peFlags = 0;
                        }

                        pPal->CreatePalette( pLP );

                        delete[] pLP;
                }
        }

        CClientDC dc(NULL);
        CPalette* pOldPalette = NULL;
        if( pPal )
        {
                pOldPalette = dc.SelectPalette( pPal, FALSE );
                dc.RealizePalette();
        }

        HBITMAP hBmp = CreateDIBitmap( dc.m_hDC,                // handle to device context 
                                &bmiHeader,     // pointer to bitmap size and format data 
                                CBM_INIT,       // initialization flag 
                                lpDIBBits,      // pointer to initialization data 
                                &bmInfo,        // pointer to bitmap color-format data 
                                DIB_RGB_COLORS);                // color-data usage 
        bitmap.Attach( hBmp );

        if( pOldPalette )
                dc.SelectPalette( pOldPalette, FALSE );

        ::GlobalFree(hDIB);
        return TRUE;
}


void DrawBitmap(CDC* pDC, CBitmap& bitmap, CPalette *pPal, CRect& rc)
{
        // Create a compatible memory DC
        CDC memDC;
        memDC.CreateCompatibleDC( pDC );
        CBitmap* pOldBmp = memDC.SelectObject( &bitmap );

        // Select and realize the palette
        CPalette* pOldPal;
        if( pPal != NULL && pDC->GetDeviceCaps(RASTERCAPS) & RC_PALETTE )
        {
                pOldPal = pDC->SelectPalette( pPal, FALSE );
                pDC->RealizePalette();
        }

        BITMAP bm;
        bitmap.GetBitmap( &bm );

        pDC->StretchBlt(rc.left, rc.top, rc.Width(), rc.Height(),
                        &memDC,
                        0, 0, bm.bmWidth, bm.bmHeight,
                        SRCCOPY);

        // Select and realize the palette
        if( pPal != NULL && pDC->GetDeviceCaps(RASTERCAPS) & RC_PALETTE )
        {
                pDC->SelectPalette( pOldPal, FALSE );
        }
        memDC.SelectObject (pOldBmp);
}


void CBmpDemoObj::Draw (CDC* pDC) {
    if (m_szBmpFile == "") {
        pDC->FillSolidRect (GetRect(), m_color);
    } else {
        if (m_bitmap.m_hObject == NULL) {
            LoadBMPImage (m_szBmpFile, m_bitmap, &m_palette);
        }

        CRect rc = GetRect();
        DrawBitmap (pDC, m_bitmap, &m_palette, rc);
    }
}


/////////////////////////////////////////////////////////////////////////////
// CDemoDoc

IMPLEMENT_DYNCREATE(CDemoDoc, CDocument)

BEGIN_MESSAGE_MAP(CDemoDoc, CDocument)
	//{{AFX_MSG_MAP(CDemoDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDemoDoc construction/destruction

CDemoDoc::CDemoDoc()
{
    // Create the objects, managed by the view.
    m_objects.AddHead(new CDemoObject (CRect(10,10,50,50), RGB(255,0,0)));
    m_objects.AddHead(new CDemoObject (CRect(70,10,100,40), RGB(0,255,0)));
    m_objects.AddHead(new CDemoObject (CRect(10,100,30,150), RGB(0,0,255)));

    m_objects.AddHead(new CPictureDemoObj (CRect(200,100,250,150), RGB(0,0,0), IDB_BEANY));
    m_objects.AddHead(new CPictureDemoObj (CRect(300,200,500,400), RGB(0,0,0), IDB_BOOKS));
    m_objects.AddHead(new CPictureDemoObj (CRect(10,100,30,150), RGB(0,0,0), IDB_SMOKES));
}

CDemoDoc::~CDemoDoc()
{
    // Garbage section
    POSITION pos = m_objects.GetHeadPosition ();
    while (pos != NULL)
        delete m_objects.GetNext (pos);
    m_objects.RemoveAll ();
}

BOOL CDemoDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CDemoDoc serialization

void CDemoDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDemoDoc diagnostics

#ifdef _DEBUG
void CDemoDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDemoDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDemoDoc commands
